var clc = require('cli-color');
var _ = require('lodash');

class Logger {

  constructor() {
  }

  /**
   * log logs obj's groups, commands (and option(s))
   */
  static log(obj) {

    /**
     * log te commands name and its help text
     */
    Logger.logName(obj);

    /**
     * log the commands synopsis
     */
    Logger.logSynopsis(obj);

    /**
     * log all command names and their help texts
     */
    if (obj._commands.length > 0) {

      console.log(clc.whiteBright.bold('COMMANDS'));

      _.each(obj._commands, function (command) {
        Logger.logCommand(command, false)
      });

    }

    /**
     * log all group names and their help texts
     */
    if (obj._groups.length > 0) {

      console.log(clc.whiteBright.bold('GROUPS'));

      _.each(obj._groups, function (group) {
        Logger.logGroup(group, false)
      });

    }

  }

  /**
   * logName logs the obj name
   */
  static logName(obj) {

    console.log(clc.whiteBright.bold('NAME'));
    console.log(`    ${obj.getFullName()} - ${obj._help}`);
    console.log();

  }

  /**
   * logFullName logs the full obj name
   */
  static logFullName(obj) {

    var path = obj.getFullName().split(' ');

    console.log(clc.whiteBright.bold('SYNOPSIS'));

    process.stdout.write('    ');
    _.each(path, function (component) {

      if (_.last(path) !== component) {
        process.stdout.write(`${clc.underline(component)} `);
      }

      if (_.last(path) == component) {
        process.stdout.write(`${clc.underline(component)}`)
      }

    });

  }

  /**
   * logSynopsis logs the synopsis of obj
   */
  static logSynopsis(obj) {

    /**
     * log the obj full name
     */
    Logger.logFullName(obj);

    /**
     * if obj is a command, log the command specific syntax
     */
    if (obj.type === 'command') {
      Logger.logCommandSynopsis(obj);
    }

    /**
     * if obj is a group, log the group specific syntax
     */
    if (obj.type === 'group') {
      Logger.logGroupSynopsis(obj);
    }

    /**
     * if obj has no type it is the root of the CLI
     */
    if (!obj.type) {
      Logger.logCLISynopsis(obj);
    }

    console.log('\n');
  }

  /**
   * logCommandSynopsis logs the synopsis for a commands
   */
  static logCommandSynopsis(command) {
    process.stdout.write(` ${clc.whiteBright.bold('<OPTIONS>')} [OPTIONS]`);
  }

  /**
   * logGroupSynopsis logs the synopsis for a group
   */
  static logGroupSynopsis(group) {

    if (group._groups.length > 0) {

      if (group._commands.length > 0) {
        process.stdout.write(` GROUP | COMMAND [OPTIONS]`);
      }

      if (group._commands.length === 0) {
        process.stdout.write(` GROUP [OPTIONS]`);
      }

    }

    if (group._groups.length === 0) {

      if (group._commands.length > 0) {
        process.stdout.write(` COMMAND [OPTIONS]`);
      }

      if (group._commands.length > 0) {
        process.stdout.write(` [OPTIONS]`);
      }

    }

  }

  /**
   * logGroupSynopsis logs the synopsis for the CLI
   */
  static logCLISynopsis(cli) {

    if (cli._groups.length > 0) {

      if (cli._commands.length > 0) {
        process.stdout.write(` GROUP | COMMAND [OPTIONS]`);
      }

      if (cli._commands.length === 0) {
        process.stdout.write(` GROUP [OPTIONS]`);
      }

    }

    if (cli._groups.length === 0) {

      if (cli._commands.length > 0) {
        process.stdout.write(` COMMAND [OPTIONS]`);
      }

      if (cli._commands.length === 0) {
        process.stdout.write(` [OPTIONS]`);
      }

    }

  }

  /**
   * logCommand logs the commands name, synopsis, description and options
   */
  static logCommand(command, detailed) {

    /**
     * if detailed is true, log the commands description, all it's options and their descriptions
     */
    if (detailed) {

      /**
       * log the commands name
       */
      Logger.logName(command);

      /**
       * log the command synopsis
       */
      Logger.logSynopsis(command, 'command');

      /**
       * log the commands description
       */
      Logger.logDescription(command, '    ', true);

      /**
       * log all options of the command in order arguments - flags
       */
      if (command._options.length > 0) {
        console.log(clc.whiteBright.bold('OPTIONS'));
        _.each(command._options, function (option) {
          Logger.logOption(option);
        });

      }

      return;
    }

    if (!detailed) {
      console.log(clc.whiteBright.bold(`    ${command._name}`));
      console.log(`        ${command._help}`);
      console.log();
    }

  }

  /**
   * logGroup logs the groups name, synopsis, description, subgroups, commands and options
   */
  static logGroup(group, detailed) {

    if (detailed) {

      /**
       * log the groups full name
       */
      Logger.logName(group);

      /**
       * if the group owns other groups, the synopsis changes
       */
      Logger.logSynopsis(group);

      /**
       * log the groups description
       */
      Logger.logDescription(group, '    ', true);

      /**
       * log the group commands
       */
      if (group._commands.length > 0) {

        console.log(clc.whiteBright.bold('COMMANDS'));

        _.each(group._commands, function (command) {
          Logger.logCommand(command, false)
        });
      }

      /**
       * log the group groups
       */
      if (group._groups.length > 0) {

        console.log(clc.whiteBright.bold('GROUPS'));

        _.each(group._groups, function (group) {
          Logger.logGroup(group, false)
        });
      }

      return;
    }

    /**
     * when group shall not be logged detailed
     * log the name and the help
     */
    if (!detailed) {
      console.log(clc.whiteBright.bold(`    ${group._name}`));
      console.log(`        ${group._help}`);
      console.log();
    }

  }

  /**
   * logOption logs the options name, its aliases and their descriptions
   */
  static logOption(flag) {

    var joinedNameAndAliases = '';

    if (flag._aliases.length > 0) {

      joinedNameAndAliases = `--${flag._name}, ` + _.map(flag._aliases, function (alias) {
          return `-${alias}`
        }).join(', ');

    }

    if (flag._aliases.length === 0) {
      joinedNameAndAliases = `--${flag._name}`;
    }

    if (flag._argument) {
      if (flag._required) {
        console.log(clc.whiteBright.bold(`    ${joinedNameAndAliases} <${flag._argument._name}>`));
      }
      else {
        console.log(`    ${joinedNameAndAliases} <${flag._argument._name}>`);
      }
    }

    if (!flag._argument) {
      if (flag._required) {
        console.log(clc.whiteBright.bold(`    ${joinedNameAndAliases}`));
      }
      else {
        console.log(`    ${joinedNameAndAliases}`);
      }
    }

    Logger.logDescription(flag, '        ', false);

  }

  /**
   *  logMissingOptions logs all options, which are required, but missing
   */
  static logMissingOptions(options) {
    process.stdout.write(clc.redBright.bold(`ERROR: `));
    process.stdout.write(clc.bold(`Missing choices: `));
    var missingOptionNames = _.map(options, function (option) {
      return option._name;
    }).join(', ');

    process.stdout.write(`'${missingOptionNames}'`);

  }

  /**
   * logDescription logs the description if object has one
   */
  static logDescription(obj, intending, headline) {

    if (!obj._description) {
      return;
    }

    if (headline) {
      console.log(clc.whiteBright.bold('DESCRIPTION'));
    }

    _.each(obj._description.split("\n"), function (line) {
      console.log(`${intending}${line}`);
    });
    console.log();

  }

}

module.exports = Logger;